/*
 * es755-access.h  --  ES755 Soc Audio access values
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef _ES755_ACCESS_H
#define _ES755_ACCESS_H


#include "es-d300.h"
#include "es-a300-reg.h"
#include "es755.h"

static struct escore_api_access es755_api_access[ES_API_ADDR_MAX] = {
	[ES_CHANGE_STATUS] = {
		.read_msg = { ES_API_WORD(0x804f, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x904f, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 4,
	},
	[ES_FW_FIRST_CHAR] = {
		.read_msg = { ES_API_WORD(0x8020, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9020, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 255,
	},
	[ES_FW_NEXT_CHAR] = {
		.read_msg = { ES_API_WORD(0x8021, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9021, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 255,
	},
	[ES_GET_SYS_INTERRUPT] = {
		.read_msg = { ES_API_WORD(0x8053, 0x0000) },
		.read_msg_len = 4,
		.val_shift = 0,
		.val_max = 0,
	},
	[ES_ACCDET_CONFIG] = {
		.read_msg = { ES_API_WORD(0x8055, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9056, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 0xFF,
	},
	[ES_GET_ACCDET_STATUS] = {
		.read_msg = { ES_API_WORD(0x8054, 0x0000) },
		.read_msg_len = 4,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_BUTTON_DETECTION_ENABLE] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f30) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f30),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_BUTTON_SERIAL_CONFIG] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f31) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f31),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_BUTTON_PARALLEL_CONFIG] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f32) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f32),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_BUTTON_DETECTION_RATE] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f33) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f33),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 3,
	},
	[ES_BUTTON_PRESS_SETTLING_TIME] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f34) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f34),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 7,
	},
	[ES_BUTTON_BOUNCE_TIME] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f35) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f35),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 11,
	},
	[ES_BUTTON_DETECTION_LONG_PRESS_TIME] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1f36) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1f36),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 15,
	},
	[ES_BUTTON_CTRL1] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1631) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1631),
			       ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_BUTTON_CTRL2] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1632) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1632),
			       ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_BUTTON_CTRL3] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1633) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1633),
			       ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_BUTTON_CTRL4] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1634) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1634),
			       ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PRIMARY_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR0, ES300_PRI)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,
	},
	[ES_SECONDARY_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR1)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR1, ES300_SEC)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR1,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_VP, IN, vp_i2)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR1, 1)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR1, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,

	},
	[ES_TERTIARY_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR2)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR2, ES300_TER)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR2,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_VP, IN, vp_i3)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR2, 2)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR2, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,
	},
	[ES_AECREF1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR4)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR4, ES300_AECREF)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR4,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_VP, IN, vp_i4)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR4, 2)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR4, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,
	},
	[ES_FEIN_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR3)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR3, ES300_FEIN)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,
	},

	[ES_UITONE1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR5)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR5, ES300_UITONE1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_UITONE2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR6)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR6, ES300_UITONE1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_UITONE3_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR5)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR5, ES300_UITONE3)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_UITONE4_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR6)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR6, ES300_UITONE4)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_MMUITONE1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR2)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR2, ES300_UITONE1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_MMUITONE2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR3)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR3, ES300_UITONE2)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_AUDIN1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR0, ES300_AUDIN1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_AUDIN2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR1)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR1, ES300_AUDIN2)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_MM_PASSIN1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR0, ES300_PASSIN1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_MM_PASSIN2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR1)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR1, ES300_PASSIN2)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR1,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
				ES300_ENDPOINT(FILTER_PASSTHRU, IN, pass_i1)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR1, 3)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR1, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_PASSIN1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR0, ES300_PASSIN1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PASSIN2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR1)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR1, ES300_PASSIN2)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR1,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
				ES300_ENDPOINT(FILTER_PASSTHRU, IN, pass_i1)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR1, 3)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR1, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PASSIN3_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR2)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR2, ES300_PASSIN3)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR2,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
				ES300_ENDPOINT(FILTER_PASSTHRU, IN, pass_i2)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR2, 4)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR2, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PASSIN4_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR3)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
				ES300_PATH_ID(RXCHMGR3, ES300_PASSIN4)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR3,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
				ES300_ENDPOINT(FILTER_PASSTHRU, IN, pass_i3)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR3, 4)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR3, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_DAC0_0_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(DAC0, 0, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_DAC0_1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(DAC0, 1, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_DAC1_0_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(DAC1, 0, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_DAC1_1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(DAC1, 1, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_PCM0_0_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM0, 0, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM0_1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM0, 1, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM0_2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM0, 2, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM0_3_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM0, 3, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM1_0_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM1, 0, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM1_1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM1, 1, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM1_2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM1, 2, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM1_3_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM1, 3, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM2_0_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM2, 0, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM2_1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM2, 1, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM2_2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM2, 2, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_PCM2_3_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(PCM2, 3, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_CODEC_VALUE] = {
		.read_msg = { ES_API_WORD(0x807B, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x907C, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_POWER_STATE] = {
		.read_msg = { ES_API_WORD(0x800f, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9010, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 6,
	},
	[ES_VS_INT_OSC_MEASURE_START] = {
		.read_msg = { ES_API_WORD(0x8070, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9070, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_VS_INT_OSC_MEASURE_STATUS] = {
		.read_msg = { ES_API_WORD(0x8071, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9071, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_EVENT_RESPONSE] = {
		.read_msg = { ES_API_WORD(0x801a, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x901a, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 4,
	},
	[ES_VOICE_SENSE_SET_KEYWORD] = {
		.read_msg = { ES_API_WORD(0x8000, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x8000, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 4,
	},
	[ES_VOICE_SENSE_EVENT] = {
		.read_msg = { ES_API_WORD(0x806d, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x8000, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 2,
	},
	[ES_VOICE_SENSE_TRAINING_MODE] = {
		.read_msg = { ES_API_WORD(ES_GET_ALGO_PARAM, 0x5003) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x5003),
				   ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 2,
	},
	[ES_VOICE_SENSE_DETECTION_SENSITIVITY] = {
		.read_msg = { ES_API_WORD(ES_GET_ALGO_PARAM, 0x5004) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x5004),
			       ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 10,
	},
	[ES_VOICE_ACTIVITY_DETECTION_SENSITIVITY] = {
		.read_msg = { ES_API_WORD(ES_GET_ALGO_PARAM, 0x5005) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x5005),
			       ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 10,
	},
	[ES_VS_STORED_KEYWORD] = {
		.read_msg = { ES_API_WORD(0x8000, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x8000, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 1,
	},
	[ES_VOICE_SENSE_TRAINING_RECORD] = {
		.read_msg = { ES_API_WORD(ES_GET_ALGO_PARAM, 0x5006) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x5006),
			       ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 5,
	},
	[ES_VOICE_SENSE_TRAINING_STATUS] = {
		.read_msg = { ES_API_WORD(ES_GET_ALGO_PARAM, 0x5007) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x5007),
			       ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 4,
	},
	[ES_VOICE_SENSE_TRAINING_MODEL_LENGTH] = {
		.read_msg = { ES_API_WORD(ES_GET_ALGO_PARAM, 0x500A) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x500A),
			       ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 75,
	},
	[ES_CVS_PRESET] = {
		.read_msg = { ES_API_WORD(0x806F, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x906F, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_SET_NS] = {
		.read_msg = { ES_API_WORD(0x8043, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x901C, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PRESET] = {
		.read_msg = { ES_API_WORD(0x8031, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9031, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_SBUSTX0_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(SBUS, 0, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},
	[ES_SBUSTX1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(SBUS, 1, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_SBUSTX2_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(SBUS, 2, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_SBUSTX3_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(SBUS, 3, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_SBUSTX4_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(SBUS, 4, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_SBUSTX5_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(SBUS, 5, 0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(0, 0)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_PATH_TYPE,
	},

	[ES_STEREO_WIDTH] = {
		.read_msg = {ES_API_WORD(ES_GET_ALGO_PARAM, 0x1006)},
		.read_msg_len = 4,
		.write_msg = {
			ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x1006),
			ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},

	[ES_MIC_CONFIG] = {
		.read_msg = {ES_API_WORD(ES_GET_ALGO_PARAM, 0x0002)},
		.read_msg_len = 4,
		.write_msg = {
			ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x0002),
			ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},

	[ES_AEC_MODE] = {
		.read_msg = {ES_API_WORD(ES_GET_ALGO_PARAM, 0x0003)},
		.read_msg_len = 4,
		.write_msg = {
			ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x0003),
			ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},

	[ES_AZ_MODE] = {
		.read_msg = {ES_API_WORD(ES_GET_ALGO_PARAM, 0x2002)},
		.read_msg_len = 4,
		.write_msg = {
			ES_API_WORD(ES_SET_ALGO_PARAM_ID, 0x2002),
			ES_API_WORD(ES_SET_ALGO_PARAM, 0x0000),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},

	[ES_PORT_WORD_LEN] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0000),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_TDM_SLOTS_PER_FRAME] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0001) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0001),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_TX_DELAY_FROM_FS] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0002) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0002),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_RX_DELAY_FROM_FS] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0003) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0003),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_LATCH_EDGE] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0004) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0004),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_ENDIAN] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0005) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0005),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_TRISTATE] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0006) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0006),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_AUDIO_MODE] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0007) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0007),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_TDM_ENABLED] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0008) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0008),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_CLOCK_CONTROL] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x0009) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x0009),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_DATA_JUSTIFICATION] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x000a) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x000a),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_PORT_FS_DURATION] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x000b) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x000b),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_AZ_PRI_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR0)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR0, ES300_PRI)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,
	},
	[ES_AZ_SEC_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR1)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR1, ES300_SEC)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,

	},
	[ES_AZ_TER_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR2)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR2, ES300_TER)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_RXCHANMGR2,
						OUT, RxChMgr_o0)),
			ES_API_WORD(ES_CONNECT_CMD,
					ES300_ENDPOINT(FILTER_AZ, IN, az_i2)),
			ES_API_WORD(ES_SET_RATE_CMD,
					ES300_RATE(FILTER_RXCHANMGR2, 3)),
			ES_API_WORD(ES_SET_GROUP_CMD,
					ES300_GROUP(FILTER_RXCHANMGR2, 0)),
		},
		.write_msg_len = 24,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,

	},
	[ES_AZ_AI1_MUX] = {
		.write_msg = {
			ES_API_WORD(ES_SET_MUX_CMD,
					ES300_DATA_PATH(0, 0, RXCHMGR3)),
			ES_API_WORD(ES_SET_PATH_ID_CMD,
					ES300_PATH_ID(RXCHMGR3, ES300_AUDIN1)),
		},
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = MAX_INPUT_PORT,
	},
	[ES_FE_STREAMING] = {
		.read_msg = { ES_API_WORD(0x8028, 0x0000) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(0x9028, 0x0000) },
		.write_msg_len = 4,
		.val_shift = 0,
		.val_max = 65535,
	},
	[ES_CODEC_OUTPUT_RATE] = {
		.read_msg = { ES_API_WORD(ES_GET_DEV_PARAM, 0x1604) },
		.read_msg_len = 4,
		.write_msg = { ES_API_WORD(ES_SET_DEV_PARAM_ID, 0x1604),
			ES_API_WORD(ES_SET_DEV_PARAM, 0x0000) },
		.write_msg_len = 8,
		.val_shift = 0,
		.val_max = 65535,
	},


};

static struct escore_reg_cache a300_reg_cache[ES_MAX_REGISTER] = {
	{ 0x01, 0 },
	{ 0x60, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0xA0, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x28, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x08, 0 },
	{ 0x00, 0 },
	{ 0x00, 1 },
	{ 0xFF, 1 },
	{ 0x00, 1 },
	{ 0xFF, 1 },
	{ 0x00, 1 },
	{ 0x00, 1 },
	{ 0x00, 1 },
	{ 0xC7, 1 },
	{ 0x00, 0 },
	{ 0x52, 0 },
	{ 0x70, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x66, 0 },
	{ 0x03, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x1F, 0 },
	{ 0x0C, 1 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 1 },
	{ 0x00, 0 },
	{ 0xFF, 0 },
	{ 0x00, 0 },
	{ 0xFF, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x00, 0 },
	{ 0x40, 0 },
	{ 0x0F, 0 },
	{ 0x41, 0 },
	{ 0xFF, 0 },
	{ 0x6B, 0 },
	{ 0x14, 0 },
	{ 0x50, 0 },
	{ 0x1D, 0 },
};


#endif /* _ES_ACCESS_H */
